<?php
class Netzkollektiv_Lexoffice_Helper_Totals extends Mage_Core_Helper_Abstract {

    protected $_defaultTotalModel = 'sales/order_pdf_total_default';

    public function getTotals($source){
        $order = $source->getOrder();
        $totals = $this->_getTotalsList($source);
        $totalsLines = array();

        foreach ($totals as $total) {
            $total->setOrder($order)
                ->setSource($source);

            if ($total->canDisplay()) {
                $amount = $total->getSource()->getDataUsingMethod('base_'.$total->getSourceField());
                $amountInclTax = $amount;

                $taxSourceField = $this->_getTaxSourceField($total->getSourceField());
                if ($taxSourceField) {
                   $amountInclTax = $amount + $total->getSource()->getDataUsingMethod('base_'.$taxSourceField);
                }

                $totalsLines[] = new Varien_Object(array(
                    'code'              => $total->getSourceField(),
                    'title'             => Mage::helper('sales')->__($total->getTitle()),
                    'amount'            => $amount,
                    'amount_incl_tax'   => $amountInclTax
                ));
            }
        }

        return $totalsLines;
    }

    protected function _getTaxSourceField($sourceField) {
        return Mage::getStoreConfig('lexoffice/tax_totals/'.$sourceField);
    }

    /**
     * Return total list
     *
     * @param  Mage_Sales_Model_Abstract $source
     * @return array
     */
    protected function _getTotalsList($source)
    {
        $totals = Mage::getConfig()->getNode('global/pdf/totals')->asArray();
        usort($totals, array($this, '_sortTotalsList'));
        $totalModels = array();
        foreach ($totals as $index => $totalInfo) {
            if (!empty($totalInfo['model'])) {
                $totalModel = Mage::getModel($totalInfo['model']);
                if ($totalModel instanceof Mage_Sales_Model_Order_Pdf_Total_Default) {
                    $totalInfo['model'] = $totalModel;
                } else {
                    Mage::throwException(
                        Mage::helper('sales')->__('PDF total model should extend Mage_Sales_Model_Order_Pdf_Total_Default')
                    );
                }
            } else {
                $totalModel = Mage::getModel($this->_defaultTotalModel);
            }
            $totalModel->setData($totalInfo);
            $totalModels[] = $totalModel;
        }

        return $totalModels;
    }

    /**
     * Sort totals list
     *
     * @param  array $a
     * @param  array $b
     * @return int
     */
    protected function _sortTotalsList($a, $b) {
        if (!isset($a['sort_order']) || !isset($b['sort_order'])) {
            return 0;
        }

        if ($a['sort_order'] == $b['sort_order']) {
            return 0;
        }

        return ($a['sort_order'] > $b['sort_order']) ? 1 : -1;
    }
}
